from __future__ import annotations
from typing import Literal


TWeightEnum = Literal["light", "regular", "medium", "bold"]
TAlignEnum = Literal["left", "center", "right"]
TTrimEnum = Literal["normal", "start", "end", "both"]
TTextWrapEnum = Literal["wrap", "nowrap", "pretty", "balance"]
