__all__ = [
    "element",
    "convert_reference",
    "configure_slot_without_slot_prop",
]

from instaui.components.element import Element as element
from ._functions import convert_reference
from ._slot import configure_slot_without_slot_prop
